#include <SoftwareSerial.h>
#include "VoiceRecognitionV3.h" /* https://github.com/elechouse/VoiceRecognitionV3 */

/**        
  Connection
  Arduino    VoiceRecognitionModule
   2   ------->     TX
   3   ------->     RX
*/
VR myVR(2,3);    // 2:RX 3:TX, you can choose your favourite pins.

uint8_t records[7]; // save record
uint8_t buf[64];

int relay1 = 5;
int relay2 = 6;

#define RELAY1_OFF    (0)
#define RELAY1_ON     (1) 
#define RELAY2_OFF    (2)
#define RELAY2_ON     (3) 

/**
  @brief   Print signature, if the character is invisible, 
           print hexible value instead.
  @param   buf     --> command length
           len     --> number of parameters
*/
void printSignature(uint8_t *buf, int len)
{
  int i;
  for(i=0; i<len; i++){
    if(buf[i]>0x19 && buf[i]<0x7F){
      Serial.write(buf[i]);
    }
    else{
      Serial.print("[");
      Serial.print(buf[i], HEX);
      Serial.print("]");
    }
  }
}

/**
  @brief   Print signature, if the character is invisible, 
           print hexible value instead.
  @param   buf  -->  VR module return value when voice is recognized.
             buf[0]  -->  Group mode(FF: None Group, 0x8n: User, 0x0n:System
             buf[1]  -->  number of record which is recognized. 
             buf[2]  -->  Recognizer index(position) value of the recognized record.
             buf[3]  -->  Signature length
             buf[4]~buf[n] --> Signature
*/
void printVR(uint8_t *buf)
{
  Serial.println("VR Index\tGroup\tRecordNum\tSignature");

  Serial.print(buf[2], DEC);
  Serial.print("\t\t");

  if(buf[0] == 0xFF){
    Serial.print("NONE");
  }
  else if(buf[0]&0x80){
    Serial.print("UG ");
    Serial.print(buf[0]&(~0x80), DEC);
  }
  else{
    Serial.print("SG ");
    Serial.print(buf[0], DEC);
  }
  Serial.print("\t");

  Serial.print(buf[1], DEC);
  Serial.print("\t\t");
  if(buf[3]>0){
    printSignature(buf+4, buf[3]);
  }
  else{
    Serial.print("NONE");
  }
  Serial.println("\r\n");
}

void setup()
{
  /** initialize */
  myVR.begin(9600);
  
  Serial.begin(115200);
  Serial.println("Elechouse Voice Recognition V3 Module\r\nControl relay sample");
  
  pinMode(relay1, OUTPUT);
  pinMode(relay2, OUTPUT);
  digitalWrite(relay1, HIGH);
  digitalWrite(relay2, HIGH);
    
  if(myVR.clear() == 0){
    Serial.println("Recognizer cleared.");
  }else{
    Serial.println("Not find VoiceRecognitionModule.");
    Serial.println("Please check connection and restart Arduino.");
    while(1);
  }
  
  if(myVR.load((uint8_t)RELAY1_OFF) >= 0){
    Serial.println("RELAY1_OFF loaded");
  }
  
  if(myVR.load((uint8_t)RELAY1_ON) >= 0){
    Serial.println("RELAY1_ON loaded");
  }

  if(myVR.load((uint8_t)RELAY2_OFF) >= 0){
    Serial.println("RELAY2_OFF loaded");
  }

  if(myVR.load((uint8_t)RELAY2_ON) >= 0){
    Serial.println("RELAY2_ON loaded");
  }
}

void loop()
{
  int ret;
  ret = myVR.recognize(buf, 50);
  if(ret>0){
    switch(buf[1]){
      case RELAY1_OFF:
        /** turn on relay 1 */
        digitalWrite(relay1, HIGH);
        break;
      case RELAY1_ON:
        /** turn off relay 1*/
        digitalWrite(relay1, LOW);
        break;
      case RELAY2_OFF:
        /** turn on relay 2*/
        digitalWrite(relay2, HIGH);
        break;
      case RELAY2_ON:
        /** turn off relay 2*/
        digitalWrite(relay2, LOW);
        break;  
      default:
        Serial.println("Record function undefined");
        break;
    }
    /** voice recognized */
    printVR(buf);
  }
}
